@@ -1,30 +1,22 @@
-/*! Representation of the `BitSlice` region memory model
+/*! Representations of the [`BitSlice`] region memory model.
 
-This module allows any `BitSlice` region to be decomposed into domains with
-more detailed aliasing information.
+This module allows any [`BitSlice`] region to be decomposed into domains that
+restricts [`T::Alias`] markers to only the edge elements that may require them.
 
-Specifically, any particular `BitSlice` region is one of:
+Specifically, any given [`BitSlice`] region is one of:
 
 - touches only interior indices of one element
 - touches at least one edge index of any number of elements (including zero)
 
-In the latter case, any elements *completely* spanned by the slice handle are
-known to not have any other write-capable views to them, and in the case of an
-`&mut BitSlice` handle specifically, no other views at all. As such, the domain
-view of this memory is able to remove the aliasing marker type and permit direct
-memory access for the duration of its existence.
-!*/
+In the latter case, any elements *completely* spanned by the [`BitSlice`] handle
+are known to not have any other write-capable handles to them, and in the case
+of an `&mut BitSlice` handle specifically, no other views at all. As such, the
+domain view of this memory is able to remove the aliasing marker type and permit
+direct memory access to the underlying buffer for the duration of its existence.
 
-use crate::{
-	index::{
-		BitIdx,
-		BitTail,
-	},
-	mem::BitMemory,
-	order::BitOrder,
-	slice::BitSlice,
-	store::BitStore,
-};
+[`BitSlice`]: crate::slice::BitSlice
+[`T::Alias`]: crate::store::BitStore::Alias
+!*/
 
 use core::{
 	fmt::{
@@ -39,31 +31,47 @@ use core::{
 	slice,
 };
 
+use funty::IsNumber;
 use tap::{
 	pipe::Pipe,
 	tap::Tap,
 };
-
 use wyz::fmt::FmtForward;
 
+use crate::{
+	index::{
+		BitIdx,
+		BitTail,
+	},
+	order::BitOrder,
+	slice::BitSlice,
+	store::BitStore,
+};
+
 macro_rules! bit_domain {
 	($t:ident $(=> $m:ident)? $(@ $a:ident)?) => {
 		/// Granular representation of the memory region containing a
-		/// `BitSlice`.
+		/// [`BitSlice`].
 		///
-		/// `BitSlice` regions can be described in terms of edge and center
-		/// elements, where the edge elements retain the aliasing status of the
-		/// source `BitSlice` handle, and the center elements are known to be
-		/// completely unaliased by any other view. This property allows any
-		/// `BitSlice` handle to be decomposed into smaller regions, and safely
-		/// remove any aliasing markers from the subregion of memory that no
-		/// longer requires them for correct access.
+		/// [`BitSlice`] regions can be described in terms of edge and center
+		/// partitions, where the edge partitions must retain the aliasing
+		/// status of the source `BitSlice` handle, and the center partition is
+		/// known to be completely unaliased by any other view. This property
+		/// allows any `BitSlice` handle to be decomposed into smaller regions,
+		/// and safely remove any aliasing markers from the center partition
+		/// that no longer requires such safeguarding.
 		///
 		/// This enum acts like the `.split*` methods in that it only subdivides
-		/// the source `BitSlice` into smaller `BitSlices`, and makes
-		/// appropriate modifications to the aliasing markers. It does not
-		/// provide references to the underlying memory elements. If you need
-		/// such access directly, use the [`Domain`] or [`DomainMut`] enums.
+		/// the source [`BitSlice`] into smaller `BitSlice`s, and makes
+		/// appropriate modifications to the aliasing markers.
+		///
+		/// It does not add any aliasing markers: if the slice is marked as
+		/// aliased, then the edge partitions will retain that marker, and if it
+		/// is not, then the edge partitions do not need it.
+		///
+		/// This does not provide references to the underlying memory buffer. If
+		/// you need such direct access, use the [`Domain`] or [`DomainMut`]
+		/// enums.
 		///
 		/// # Lifetimes
 		///
@@ -71,47 +79,59 @@ macro_rules! bit_domain {
 		///
 		/// # Type Parameters
 		///
-		/// - `O`: The ordering type of the source `BitSlice` handle.
-		/// - `T`: The element type of the source `BitSlice` handle, including
-		///   aliasing markers.
+		/// - `O`: The ordering type of the source [`BitSlice`] handle.
+		/// - `T`: The register type of the source [`BitSlice`] handle,
+		///   including any aliasing markers.
 		///
 		/// # Aliasing Awareness
 		///
 		/// This enum does not grant access to memory outside the scope of the
-		/// original `&BitSlice` handle, and so does not need to modfiy any
+		/// original [`BitSlice`] handle, and so does not need to modfiy any
 		/// aliasing conditions.
 		///
-		/// [`Domain`]: enum.Domain.html
-		/// [`DomainMut`]: enum.DomainMut.html
+		/// [`BitSlice`]: crate::slice::BitSlice
+		/// [`Domain`]: crate::domain::Domain
+		/// [`DomainMut`]: crate::domain::DomainMut
 		#[derive(Debug)]
 		pub enum $t <'a, O, T>
 		where
 			O: BitOrder,
-			T: 'a + BitStore
+			T: BitStore,
 		{
-			/// Indicates that a `BitSlice` is contained entirely in the
-			/// interior indices of a single memory element.
+			/// Indicates that a [`BitSlice`] is contained entirely in the
+			/// interior indices of a single memory register.
+			///
+			/// [`BitSlice`]: crate::slice::BitSlice
 			Enclave {
-				/// The start index of the `BitSlice`.
+				/// The start index of the [`BitSlice`].
 				///
 				/// This is not likely to be useful information, but is retained
 				/// for structural similarity with the rest of the module.
+				///
+				/// [`BitSlice`]: crate::slice::BitSlice
 				head: BitIdx<T::Mem>,
-				/// The original `BitSlice` used to create this bit-domain view.
+				/// The original [`BitSlice`] used to create this bit-domain
+				/// view.
+				///
+				/// [`BitSlice`]: crate::slice::BitSlice
 				body: &'a $($m)? BitSlice<O, T>,
-				/// The end index of the `BitSlice`.
+				/// The end index of the [`BitSlice`].
 				///
 				/// This is not likely to be useful information, but is retained
 				/// for structural similarity with the rest of the module.
+				///
+				/// [`BitSlice`]: crate::slice::BitSlice
 				tail: BitTail<T::Mem>,
 			},
-			/// Indicates that a `BitSlice` region touches at least one edge
+			/// Indicates that a [`BitSlice`] region touches at least one edge
 			/// index of any number of elements.
 			///
-			/// This contains two bitslices representing the partially-occupied
-			/// edge elements, with their original aliasing marker, and one
-			/// bitslice representing the fully-occupied interior elements,
-			/// marked as unaliased.
+			/// This contains two [`BitSlice`]s representing the
+			/// partially-occupied edge elements, with their original aliasing
+			/// marker, and one `BitSlice` representing the fully-occupied
+			/// interior elements, marked as unaliased.
+			///
+			/// [`BitSlice`]: crate::slice::BitSlice
 			Region {
 				/// Any bits that partially-fill the base element of the slice
 				/// region.
@@ -119,15 +139,17 @@ macro_rules! bit_domain {
 				/// This does not modify its aliasing status, as it will already
 				/// be appropriately marked before constructing this view.
 				head: &'a $($m)? BitSlice<O, T>,
-				/// Any bits inside elements that the source bitslice completely
-				/// covers.
+				/// Any bits inside elements that the source [`BitSlice`]
+				/// completely covers.
 				///
 				/// This is marked as unaliased, because it is statically
 				/// impossible for any other handle to have write access to the
-				/// region it covers. As such, a bitslice that was marked as
+				/// region it covers. As such, a [`BitSlice`] that was marked as
 				/// entirely aliased, but contains interior unaliased elements,
 				/// can safely remove its aliasing protections.
-				body: &'a $($m)? BitSlice<O, T::Mem>,
+				///
+				/// [`BitSlice`]: crate::slice::BitSlice
+				body: &'a $($m)? BitSlice<O, T::Unalias>,
 				/// Any bits that partially fill the last element of the slice
 				/// region.
 				///
@@ -140,7 +162,7 @@ macro_rules! bit_domain {
 		impl<'a, O, T> $t <'a, O, T>
 		where
 			O: BitOrder,
-			T: 'a + BitStore,
+			T: BitStore,
 		{
 			/// Attempts to view the domain as an enclave variant.
 			///
@@ -153,7 +175,7 @@ macro_rules! bit_domain {
 			/// If `self` is the [`Enclave`] variant, this returns `Some` of the
 			/// enclave fields, as a tuple. Otherwise, it returns `None`.
 			///
-			/// [`Enclave`]: #variant.Enclave
+			/// [`Enclave`]: Self::Enclave
 			#[inline]
 			pub fn enclave(self) -> Option<(
 				BitIdx<T::Mem>,
@@ -179,11 +201,11 @@ macro_rules! bit_domain {
 			/// If `self` is the [`Region`] variant, this returns `Some` of the
 			/// region fields, as a tuple. Otherwise, it returns `None`.
 			///
-			/// [`Region`]: #variant.Region
+			/// [`Region`]: Self::Region
 			#[inline]
 			pub fn region(self) -> Option<(
 				&'a $($m)? BitSlice<O, T>,
-				&'a $($m)? BitSlice<O, T::Mem>,
+				&'a $($m)? BitSlice<O, T::Unalias>,
 				&'a $($m)? BitSlice<O, T>,
 			)> {
 				if let Self::Region { head, body, tail } = self {
@@ -194,23 +216,26 @@ macro_rules! bit_domain {
 				}
 			}
 
-			/// Constructs a bit-domain view from a bitslice.
+			/// Constructs a bit-domain view from a [`BitSlice`].
 			///
 			/// # Parameters
 			///
-			/// - `slice`: The source bitslice for which the view is constructed
+			/// - `slice`: The source [`BitSlice`] for which the view is
+			///   constructed.
 			///
 			/// # Returns
 			///
 			/// A bit-domain view over the source slice.
+			///
+			/// [`BitSlice`]: crate::slice::BitSlice
 			#[inline]
 			pub(crate) fn new(slice: &'a $($m)? BitSlice<O, T>) -> Self {
-				let bitptr = slice.bitptr();
-				let h = bitptr.head();
-				let (e, t) = h.span(bitptr.len());
-				let w = T::Mem::BITS;
+				let bitspan = slice.as_bitspan();
+				let h = bitspan.head();
+				let (e, t) = h.span(bitspan.len());
+				let w = T::Mem::BITS as u8;
 
-				match (h.value(), e, t.value()) {
+				match (h.into_inner(), e, t.into_inner()) {
 					(_, 0, _) => Self::empty(),
 					(0, _, t) if t == w => Self::spanning(slice),
 					(_, _, t) if t == w => Self::partial_head(slice, h),
@@ -220,7 +245,7 @@ macro_rules! bit_domain {
 				}
 			}
 
-			#[inline]
+			#[cfg_attr(not(tarpaulin_include), inline(always))]
 			fn empty() -> Self {
 				Self::Region {
 					head: Default::default(),
@@ -237,11 +262,11 @@ macro_rules! bit_domain {
 			) -> Self {
 				let (head, rest) = bit_domain!(split $($m)?
 					slice,
-					(T::Mem::BITS - head.value()) as usize,
+					(T::Mem::BITS as u8 - head.into_inner()) as usize,
 				);
 				let (body, tail) = bit_domain!(split $($m)?
 					rest,
-					rest.len() - (tail.value() as usize),
+					rest.len() - (tail.into_inner() as usize),
 				);
 				Self::Region {
 					head: bit_domain!(retype $($m)? head),
@@ -270,7 +295,7 @@ macro_rules! bit_domain {
 			) -> Self {
 				let (head, rest) = bit_domain!(split $($m)?
 					slice,
-					(T::Mem::BITS - head.value()) as usize,
+					(T::Mem::BITS as u8 - head.into_inner()) as usize,
 				);
 				let (head, body) = (
 					bit_domain!(retype $($m)? head),
@@ -296,7 +321,7 @@ macro_rules! bit_domain {
 			) -> Self {
 				let (rest, tail) = bit_domain!(split $($m)?
 					slice,
-					slice.len() - (tail.value() as usize),
+					slice.len() - (tail.into_inner() as usize),
 				);
 				let (body, tail) = (
 					bit_domain!(retype $($m)? rest),
@@ -309,7 +334,7 @@ macro_rules! bit_domain {
 				}
 			}
 
-			#[inline]
+			#[cfg_attr(not(tarpaulin_include), inline(always))]
 			fn spanning(slice: &'a $($m)? BitSlice<O, T>) -> Self {
 				Self::Region {
 					head: Default::default(),
@@ -360,21 +385,21 @@ where
 macro_rules! domain {
 	($t:ident $(=> $m:ident @ $a:ident)?) => {
 		/// Granular representation of the memory region containing a
-		/// `BitSlice`.
+		/// [`BitSlice`].
 		///
-		/// `BitSlice` regions can be described in terms of edge and center
-		/// elements, where the edge elements retain the aliasing status of the
-		/// source `BitSlice` handle, and the center elements are known to be
-		/// completely unaliased by any other view. This property allows any
-		/// `BitSlice` handle to be decomposed into smaller regions, and safely
-		/// remove any aliasing markers from the subregion of memory that no
-		/// longer requires them for correct access.
+		/// [`BitSlice`] regions can be described in terms of edge and center
+		/// partitions, where the edge partitions must retain the aliasing
+		/// status of the source `BitSlice` handle, and the center partition is
+		/// known to be completely unaliased by any other view. This property
+		/// allows any `BitSlice` handle to be decomposed into smaller regions,
+		/// and safely remove any aliasing markers from the center partition
+		/// that no longer requires such safeguarding.
 		///
-		/// This enum splits the element region backing a `BitSlice` into
-		/// maybe-aliased and known-unaliased subslices. If you do not need to
-		/// work directly with the memory elements, and only need to firmly
-		/// specify the aliasing status of a `BitSlice`, see the [`BitDomain`]
-		/// and [`BitDomainMut`] enums.
+		/// This enum splits the underlying element slice `[T]` into the
+		/// maybe-aliased edge elements and known-unaliased center elements. If
+		/// you do not need to work with the memory elements directly, and only
+		/// need to firmly specify the access behavior of the [`BitSlice`]
+		/// handle, use the [`BitDomain`] and [`BitDomainMut`] enums.
 		///
 		/// # Lifetimes
 		///
@@ -382,63 +407,80 @@ macro_rules! domain {
 		///
 		/// # Type Parameters
 		///
-		/// - `T`: The element type of the source `BitSlice` handle, including
-		///   aliasing markers.
+		/// - `T`: The register type of the source [`BitSlice`] handle,
+		///   including any aliasing markers.
 		///
-		/// [`BitDomain`]: enum.BitDomain.html
-		/// [`BitDomainMut`]: enum.BitDomainMut.html
+		/// # Mutability
+		///
+		/// The immutable view produces [`T::Alias`] references, which permit
+		/// foreign writes to the referent location but disallow writes through
+		/// itself. The mutable view produces [`T::Access`] references, because
+		/// `&mut _` references can only ever be produced when no other aliasing
+		/// handle exists. The write permissions must be weakened from
+		/// `&mut T::Alias` to `&T::Access` in order to satisfy the Rust memory
+		/// rules.
+		///
+		/// The edge references do not forbid modifying bits outside of the
+		/// source [`BitSlice`] domain, and writes out of bounds will be
+		/// correctly handled by any other handles capable of viewing those
+		/// elements. Doing so is still *incorrect*, though defined, and you are
+		/// responsible for writing only within bounds when using `DomainMut`.
+		///
+		/// [`BitDomain`]: crate::domain::BitDomain
+		/// [`BitDomainMut`]: crate::domain::BitDomainMut
+		/// [`BitSlice`]: crate::slice::BitSlice
+		/// [`T::Access`]: crate::store::BitStore::Access
+		/// [`T::Alias`]: crate::store::BitStore::Alias
 		#[derive(Debug)]
 		pub enum $t <'a, T>
 		where
-			T: 'a + BitStore,
+			T: BitStore,
 		{
-			/// Indicates that a `BitSlice` is contained entirely in the
+			/// Indicates that a [`BitSlice`] is contained entirely in the
 			/// interior indices of a single memory element.
+			///
+			/// [`BitSlice`]: crate::slice::BitSlice
 			Enclave {
-				/// The start index of the `BitSlice`.
+				/// The start index of the [`BitSlice`].
+				///
+				/// [`BitSlice`]: crate::slice::BitSlice
 				head: BitIdx<T::Mem>,
-				/// An aliased view of the element containing the `BitSlice`.
+				/// An aliased view of the element containing the [`BitSlice`].
 				///
 				/// This is necessary even on immutable views, because other
 				/// views to the referent element may be permitted to modify it.
+				///
+				/// [`BitSlice`]: crate::slice::BitSlice
 				elem: &'a T $(::$a)?,
-				/// The end index of the `BitSlice`.
+				/// The end index of the [`BitSlice`].
+				///
+				/// [`BitSlice`]: crate::slice::BitSlice
 				tail: BitTail<T::Mem>,
 			},
-			/// Indicates that a `BitSlice` region touches at least one edge
+			/// Indicates that a [`BitSlice`] region touches at least one edge
 			/// index of any number of elements.
 			///
 			/// This contains two optional references to the aliased edges, and
 			/// one reference to the unaliased middle. Each can be queried and
 			/// used individually.
+			///
+			/// [`BitSlice`]: crate::slice::BitSlice
 			Region {
-				/// If the `BitSlice` started in the interior of its first
+				/// If the [`BitSlice`] started in the interior of its first
 				/// element, this contains the starting index and the base
 				/// address.
+				///
+				/// [`BitSlice`]: crate::slice::BitSlice
 				head: Option<(BitIdx<T::Mem>, &'a T $(::$a)?)>,
 				/// All fully-spanned, unaliased, elements.
 				///
 				/// This is marked as bare memory without any access
 				/// protections, because it is statically impossible for any
 				/// other handle to have write access to the region it covers.
-				/// As such, a bitslice that was marked as entirely aliased, but
-				/// contains interior unaliased elements, can safely remove its
-				/// aliasing protections.
-				///
-				/// # Safety Exception
-				///
-				/// `&BitSlice<O, T::Alias>` references have access to a
-				/// `.set_aliased` method, which represents the only means in
-				/// `bitvec` of writing to memory without an exclusive `&mut `
-				/// reference.
-				///
-				/// Construction of two such shared, aliasing, references over
-				/// the same data, then construction of a domain view over one
-				/// of them and simultaneous writing through the other to
-				/// interior elements marked as unaliased, will cause the domain
-				/// view to be undefined behavior. Do not combine domain views
-				/// and `.set_aliased` calls.
-				body: &'a $($m)? [T::Mem],
+				/// As such, a [`BitSlice`] that was marked as entirely aliased,
+				/// but contains interior unaliased elements, can safely remove
+				/// its aliasing protections.
+				body: &'a $($m)? [T::Unalias],
 				/// If the `BitSlice` ended in the interior of its last element,
 				/// this contains the ending index and the last address.
 				tail: Option<(&'a T $(::$a)?, BitTail<T::Mem>)>,
@@ -447,7 +489,7 @@ macro_rules! domain {
 
 		impl<'a, T> $t <'a, T>
 		where
-			T: 'a + BitStore,
+			T: BitStore,
 		{
 			/// Attempts to view the domain as an enclave variant.
 			///
@@ -460,7 +502,7 @@ macro_rules! domain {
 			/// If `self` is the [`Enclave`] variant, this returns `Some` of the
 			/// enclave fields, as a tuple. Otherwise, it returns `None`.
 			///
-			/// [`Enclave`]: #variant.Enclave
+			/// [`Enclave`]: Self::Enclave
 			#[inline]
 			pub fn enclave(self) -> Option<(
 				BitIdx<T::Mem>,
@@ -485,11 +527,11 @@ macro_rules! domain {
 			/// If `self` is the [`Region`] variant, this returns `Some` of the
 			/// region fields, as a tuple. Otherwise, it returns `None`.
 			///
-			/// [`Region`]: #variant.Region
+			/// [`Region`]: Self::Region
 			#[inline]
 			pub fn region(self) -> Option<(
 				Option<(BitIdx<T::Mem>, &'a T $(::$a)?)>,
-				&'a $($m)? [T::Mem],
+				&'a $($m)? [T::Unalias],
 				Option<(&'a T $(::$a)?, BitTail<T::Mem>)>,
 			)> {
 				if let Self::Region { head, body, tail } = self {
@@ -503,13 +545,13 @@ macro_rules! domain {
 			#[inline]
 			pub(crate) fn new<O>(slice: &'a $($m)? BitSlice<O, T>) -> Self
 			where O: BitOrder {
-				let bitptr = slice.bitptr();
-				let head = bitptr.head();
-				let elts = bitptr.elements();
-				let tail = bitptr.tail();
-				let bits = T::Mem::BITS;
-				let base = bitptr.pointer().to_const() as *const _;
-				match (head.value(), elts, tail.value()) {
+				let bitspan = slice.as_bitspan();
+				let head = bitspan.head();
+				let elts = bitspan.elements();
+				let tail = bitspan.tail();
+				let bits = T::Mem::BITS as u8;
+				let base = bitspan.address().to_const() as *const _;
+				match (head.into_inner(), elts, tail.into_inner()) {
 					(_, 0, _) => Self::empty(),
 					(0, _, t) if t == bits => Self::spanning(base, elts),
 					(_, _, t) if t == bits => Self::partial_head(base, elts, head),
@@ -519,7 +561,7 @@ macro_rules! domain {
 				}
 			}
 
-			#[inline]
+			#[cfg_attr(not(tarpaulin_include), inline(always))]
 			fn empty() -> Self {
 				Self::Region {
 					head: None,
@@ -588,7 +630,7 @@ macro_rules! domain {
 				}
 			}
 
-			#[inline]
+			#[cfg_attr(not(tarpaulin_include), inline(always))]
 			fn spanning(base: *const T $(::$a)?, elts: usize) -> Self {
 				Self::Region {
 					head: None,
@@ -608,7 +650,7 @@ macro_rules! domain {
 }
 
 domain!(Domain);
-domain!(DomainMut => mut @ Alias);
+domain!(DomainMut => mut @ Access);
 
 #[cfg(not(tarpaulin_include))]
 impl<T> Clone for Domain<'_, T>
@@ -621,11 +663,10 @@ where T: BitStore
 }
 
 impl<'a, T> Iterator for Domain<'a, T>
-where T: 'a + BitStore
+where T: BitStore
 {
 	type Item = T::Mem;
 
-	#[inline]
 	fn next(&mut self) -> Option<Self::Item> {
 		match self {
 			Self::Enclave { elem, .. } => {
@@ -637,7 +678,7 @@ where T: 'a + BitStore
 				}
 				if let Some((elem, rest)) = body.split_first() {
 					*body = rest;
-					return Some(*elem);
+					return elem.load_value().into();
 				}
 				if let Some((elem, _)) = *tail {
 					return elem.load_value().pipe(Some).tap(|_| *tail = None);
@@ -649,9 +690,8 @@ where T: 'a + BitStore
 }
 
 impl<'a, T> DoubleEndedIterator for Domain<'a, T>
-where T: 'a + BitStore
+where T: BitStore
 {
-	#[inline]
 	fn next_back(&mut self) -> Option<Self::Item> {
 		match self {
 			Self::Enclave { elem, .. } => {
@@ -663,7 +703,7 @@ where T: 'a + BitStore
 				}
 				if let Some((elem, rest)) = body.split_last() {
 					*body = rest;
-					return Some(*elem);
+					return elem.load_value().into();
 				}
 				if let Some((_, elem)) = *head {
 					return elem.load_value().pipe(Some).tap(|_| *head = None);
