@@ -85,7 +85,7 @@ index-to-position.
 /// pub struct HiLo;
 /// unsafe impl BitOrder for HiLo {
 ///   fn at<R: BitRegister>(idx: BitIdx<R>) -> BitPos<R> {
-///     BitPos::new(idx.value() ^ 4).unwrap()
+///     BitPos::new(idx.into_inner() ^ 4).unwrap()
 ///   }
 /// }
 ///
@@ -202,7 +202,9 @@ pub unsafe trait BitOrder: 'static {
 	///
 	/// [`BitIdx`]: crate::index::BitIdx
 	/// [`Self::at`]: Self::at
+	#[inline]
 	#[cfg(not(tarpaulin_include))]
+
 	fn select<R>(index: BitIdx<R>) -> BitSel<R>
 	where R: BitRegister {
 		Self::at::<R>(index).select()
@@ -262,16 +264,19 @@ pub unsafe trait BitOrder: 'static {
 pub struct Lsb0;
 
 unsafe impl BitOrder for Lsb0 {
+	#[cfg_attr(not(tarpaulin_include), inline(always))]
 	fn at<R>(index: BitIdx<R>) -> BitPos<R>
 	where R: BitRegister {
-		unsafe { BitPos::new_unchecked(index.value()) }
+		unsafe { BitPos::new_unchecked(index.into_inner()) }
 	}
 
+	#[cfg_attr(not(tarpaulin_include), inline(always))]
 	fn select<R>(index: BitIdx<R>) -> BitSel<R>
 	where R: BitRegister {
-		unsafe { BitSel::new_unchecked(R::ONE << index.value()) }
+		unsafe { BitSel::new_unchecked(R::ONE << index.into_inner()) }
 	}
 
+	#[inline]
 	fn mask<R>(
 		from: impl Into<Option<BitIdx<R>>>,
 		upto: impl Into<Option<BitTail<R>>>,
@@ -279,8 +284,8 @@ unsafe impl BitOrder for Lsb0 {
 	where
 		R: BitRegister,
 	{
-		let from = from.into().unwrap_or(BitIdx::ZERO).value();
-		let upto = upto.into().unwrap_or(BitTail::LAST).value();
+		let from = from.into().unwrap_or(BitIdx::ZERO).into_inner();
+		let upto = upto.into().unwrap_or(BitTail::LAST).into_inner();
 		debug_assert!(
 			from <= upto,
 			"Ranges must run from low index ({}) to high ({})",
@@ -288,7 +293,7 @@ unsafe impl BitOrder for Lsb0 {
 			upto
 		);
 		let ct = upto - from;
-		if ct == R::BITS {
+		if ct == R::BITS as u8 {
 			return BitMask::ALL;
 		}
 		//  1. Set all bits in the mask high
@@ -305,11 +310,13 @@ unsafe impl BitOrder for Lsb0 {
 pub struct Msb0;
 
 unsafe impl BitOrder for Msb0 {
+	#[cfg_attr(not(tarpaulin_include), inline(always))]
 	fn at<R>(index: BitIdx<R>) -> BitPos<R>
 	where R: BitRegister {
-		unsafe { BitPos::new_unchecked(R::MASK - index.value()) }
+		unsafe { BitPos::new_unchecked(R::MASK - index.into_inner()) }
 	}
 
+	#[cfg_attr(not(tarpaulin_include), inline(always))]
 	fn select<R>(index: BitIdx<R>) -> BitSel<R>
 	where R: BitRegister {
 		/* Shift the MSbit down by the index count. This is not equivalent to
@@ -317,9 +324,10 @@ unsafe impl BitOrder for Msb0 {
 		subtraction followed by a rshift, while this lowers to a single rshift.
 		*/
 		let msbit: R = R::ONE << R::MASK;
-		unsafe { BitSel::new_unchecked(msbit >> index.value()) }
+		unsafe { BitSel::new_unchecked(msbit >> index.into_inner()) }
 	}
 
+	#[inline]
 	fn mask<R>(
 		from: impl Into<Option<BitIdx<R>>>,
 		upto: impl Into<Option<BitTail<R>>>,
@@ -327,8 +335,8 @@ unsafe impl BitOrder for Msb0 {
 	where
 		R: BitRegister,
 	{
-		let from = from.into().unwrap_or(BitIdx::ZERO).value();
-		let upto = upto.into().unwrap_or(BitTail::LAST).value();
+		let from = from.into().unwrap_or(BitIdx::ZERO).into_inner();
+		let upto = upto.into().unwrap_or(BitTail::LAST).into_inner();
 		debug_assert!(
 			from <= upto,
 			"Ranges must run from low index ({}) to high ({})",
@@ -336,7 +344,7 @@ unsafe impl BitOrder for Msb0 {
 			upto
 		);
 		let ct = upto - from;
-		if ct == R::BITS {
+		if ct == R::BITS as u8 {
 			return BitMask::ALL;
 		}
 		//  1. Set all bits in the mask high.
@@ -357,7 +365,6 @@ otherwise completely unrelated.
 **/
 #[cfg(target_endian = "little")]
 pub use self::Lsb0 as LocalBits;
-
 /** A default bit ordering.
 
 Typically, your platform’s C compiler uses most-significant-bit-first ordering
@@ -398,6 +405,7 @@ for `O`.
 [`BitOrder`]: crate::order::BitOrder
 [`BitRegister`]: crate::mem::BitRegister
 **/
+#[inline(never)]
 pub fn verify<O>(verbose: bool)
 where O: BitOrder {
 	verify_for_type::<O, u8>(verbose);
@@ -436,6 +444,7 @@ for the combination of input types and index values.
 [`BitOrder`]: crate::order::BitOrder
 [`BitRegister`]: crate::mem::BitRegister
 **/
+#[inline(never)]
 pub fn verify_for_type<O, R>(verbose: bool)
 where
 	O: BitOrder,
@@ -447,7 +456,7 @@ where
 	let oname = type_name::<O>();
 	let mname = type_name::<R>();
 
-	for n in 0 .. R::BITS {
+	for n in 0 .. R::BITS as u8 {
 		//  Wrap the counter as an index.
 		let idx = unsafe { BitIdx::<R>::new_unchecked(n) };
 
@@ -460,19 +469,19 @@ where
 				oname,
 				mname,
 				n,
-				pos.value(),
+				pos.into_inner(),
 			);
 		}
 
 		//  If the computed position exceeds the valid range, fail.
 		assert!(
-			pos.value() < R::BITS,
+			pos.into_inner() < R::BITS as u8,
 			"Error when verifying the implementation of `BitOrder` for `{}`: \
 			 Index {} produces a bit position ({}) that exceeds the type width \
 			 {}",
 			oname,
 			n,
-			pos.value(),
+			pos.into_inner(),
 			R::BITS,
 		);
 
@@ -488,7 +497,7 @@ where
 
 		//  If the selector bit is not one-hot, fail.
 		assert_eq!(
-			sel.value().count_ones(),
+			sel.into_inner().count_ones(),
 			1,
 			"Error when verifying the implementation of `BitOrder` for `{}`: \
 			 Index {} produces a bit selector ({:b}) that is not a one-hot mask",
@@ -510,7 +519,7 @@ where
 			oname,
 			n,
 			sel,
-			pos.value(),
+			pos.into_inner(),
 			shl,
 		);
 
@@ -523,7 +532,7 @@ where
 			 produced by a prior index",
 			oname,
 			n,
-			pos.value(),
+			pos.into_inner(),
 		);
 		accum.insert(sel);
 		if verbose {
